@@ -162,7 +162,7 @@ class FileAdmin(BaseView, ActionsMixin):
             :param base_url:
                 Base URL for the files
             :param name:
-                Name of this view. If not provided, will be defaulted to the class name.
+                Name of this view. If not provided, will default to the class name.
             :param category:
                 View category
             :param endpoint:
@@ -171,7 +171,7 @@ class FileAdmin(BaseView, ActionsMixin):
                 URL for view
             :param verify_path:
                 Verify if path exists. If set to `True` and path does not exist
-                will throw exception.
+                will raise an exception.
         """
         self.base_path = base_path
         self.base_url = base_url
@@ -198,7 +198,7 @@ class FileAdmin(BaseView, ActionsMixin):
 
     def is_accessible_path(self, path):
         """
-            Verify if path is accessible for current user.
+            Verify if the provided path is accessible for the current user.
 
             Override to customize behavior.
 
@@ -242,7 +242,7 @@ class FileAdmin(BaseView, ActionsMixin):
 
     def is_file_editable(self, filename):
         """
-            Verify if file can be edited.
+            Determine if the file can be edited.
 
             Override to customize behavior.
 
@@ -254,14 +254,14 @@ class FileAdmin(BaseView, ActionsMixin):
         if ext.startswith('.'):
             ext = ext[1:]
 
-        if self.editable_extensions and ext not in self.editable_extensions:
+        if not self.editable_extensions or ext not in self.editable_extensions:
             return False
 
         return True
 
     def is_in_folder(self, base_path, directory):
         """
-            Verify if `directory` is in `base_path` folder
+            Verify that `directory` is in `base_path` folder
 
             :param base_path:
                 Base directory path
@@ -319,9 +319,9 @@ class FileAdmin(BaseView, ActionsMixin):
         """
             Verify and normalize path.
 
-            If path is not relative to the base directory, will throw 404 exception.
+            If the path is not relative to the base directory, will raise a 404 exception.
 
-            If path does not exist, will also throw 404 exception.
+            If the path does not exist, this will also raise a 404 exception.
         """
         base_path = self.get_base_path()
 
@@ -346,6 +346,66 @@ class FileAdmin(BaseView, ActionsMixin):
 
         return True
 
+    def on_rename(self, full_path, dir_base, filename):
+        """
+            Perform some actions after a file or directory has been renamed.
+
+            Called from rename method
+
+            By default do nothing.
+        """
+        pass
+
+    def on_edit_file(self, full_path, path):
+        """
+            Perform some actions after a file has been successfully changed.
+
+            Called from edit method
+
+            By default do nothing.
+        """
+        pass
+
+    def on_file_upload(self, directory, path, filename):
+        """
+            Perform some actions after a file has been successfully uploaded.
+
+            Called from upload method
+
+            By default do nothing.
+        """
+        pass
+
+    def on_mkdir(self, parent_dir, dir_name):
+        """
+            Perform some actions after a directory has successfully been created.
+
+            Called from mkdir method
+
+            By default do nothing.
+        """
+        pass
+
+    def on_directory_delete(self, full_path, dir_name):
+        """
+            Perform some actions after a directory has successfully been deleted.
+
+            Called from delete method
+
+            By default do nothing.
+        """
+        pass
+
+    def on_file_delete(self, full_path, filename):
+        """
+            Perform some actions after a file has successfully been deleted.
+
+            Called from delete method
+
+            By default do nothing.
+        """
+        pass
+
     @expose('/')
     @expose('/b/<path:path>')
     def index(self, path=None):
@@ -353,7 +413,7 @@ class FileAdmin(BaseView, ActionsMixin):
             Index view method
 
             :param path:
-                Optional directory path. If not provided, will use base directory
+                Optional directory path. If not provided, will use the base directory
         """
         # Get path and verify if it is valid
         base_path, directory, path = self._normalize_path(path)
@@ -374,6 +434,9 @@ class FileAdmin(BaseView, ActionsMixin):
 
             items.append((f, op.join(path, f), op.isdir(fp), op.getsize(fp)))
 
+        # Sort by name
+        items.sort(key=itemgetter(0))
+
         # Sort by type
         items.sort(key=itemgetter(2), reverse=True)
 
@@ -403,7 +466,7 @@ class FileAdmin(BaseView, ActionsMixin):
             Upload view method
 
             :param path:
-                Optional directory path. If not provided, will use base directory
+                Optional directory path. If not provided, will use the base directory
         """
         # Get path and verify if it is valid
         base_path, directory, path = self._normalize_path(path)
@@ -418,11 +481,12 @@ class FileAdmin(BaseView, ActionsMixin):
                                secure_filename(form.upload.data.filename))
 
             if op.exists(filename):
-                flash(gettext('File "%(name)s" already exists.', name=form.upload.data.filename),
+                flash(gettext('File "%(name)s" already exists.', name=filename),
                       'error')
             else:
                 try:
                     self.save_file(filename, form.upload.data)
+                    self.on_file_upload(directory, path, filename)
                     return redirect(self._get_dir_url('.index', path))
                 except Exception, ex:
                     flash(gettext('Failed to save file: %(error)s', error=ex))
@@ -436,7 +500,7 @@ class FileAdmin(BaseView, ActionsMixin):
             Directory creation view method
 
             :param path:
-                Optional directory path. If not provided, will use base directory
+                Optional directory path. If not provided, will use the base directory
         """
         # Get path and verify if it is valid
         base_path, directory, path = self._normalize_path(path)
@@ -452,6 +516,7 @@ class FileAdmin(BaseView, ActionsMixin):
         if form.validate_on_submit():
             try:
                 os.mkdir(op.join(directory, form.name.data))
+                self.on_mkdir(directory, form.name.data)
                 return redirect(dir_url)
             except Exception, ex:
                 flash(gettext('Failed to create directory: %(error)s', ex), 'error')
@@ -486,12 +551,14 @@ class FileAdmin(BaseView, ActionsMixin):
 
             try:
                 shutil.rmtree(full_path)
+                self.on_directory_delete(full_path, path)
                 flash(gettext('Directory "%s" was successfully deleted.' % path))
             except Exception, ex:
                 flash(gettext('Failed to delete directory: %(error)s', error=ex), 'error')
         else:
             try:
                 os.remove(full_path)
+                self.on_file_delete(full_path, path)
                 flash(gettext('File "%(name)s" was successfully deleted.', name=path))
             except Exception, ex:
                 flash(gettext('Failed to delete file: %(name)s', name=ex), 'error')
@@ -527,6 +594,7 @@ class FileAdmin(BaseView, ActionsMixin):
                 filename = secure_filename(form.name.data)
 
                 os.rename(full_path, op.join(dir_base, filename))
+                self.on_rename(full_path, dir_base, filename)
                 flash(gettext('Successfully renamed "%(src)s" to "%(dst)s"',
                       src=op.basename(path),
                       dst=filename))
@@ -572,6 +640,7 @@ class FileAdmin(BaseView, ActionsMixin):
                     flash(gettext("Error saving changes to %(name)s.", name=path), 'error')
                     error = True
                 else:
+                    self.on_edit_file(full_path, path)
                     flash(gettext("Changes to %(name)s saved successfully.", name=path))
                     return redirect(next_url)
         else:
@@ -597,7 +666,7 @@ class FileAdmin(BaseView, ActionsMixin):
                     form.content.data = content
 
         return self.render(self.edit_template, dir_url=dir_url, path=path,
-                        form=form, error=error)
+                           form=form, error=error)
 
     @expose('/action/', methods=('POST',))
     def action_view(self):
