@@ -1,12 +1,11 @@
 import time
 import datetime
 
-from flask.globals import _request_ctx_stack
-
-from flask.ext import wtf
 from wtforms import fields, widgets
-
+from flask.globals import _request_ctx_stack
+from flask.ext import wtf
 from flask.ext.admin.babel import gettext, ngettext
+from flask.ext.admin import helpers as h
 
 
 class BaseForm(wtf.Form):
@@ -25,6 +24,7 @@ class BaseForm(wtf.Form):
     def has_file_field(self):
         """
             Return True if form contains at least one FileField.
+            Does not check for child form fields.
         """
         # TODO: Optimize me
         for f in self:
@@ -110,6 +110,45 @@ class Select2Field(fields.SelectField):
     """
     widget = Select2Widget()
 
+    def __init__(self, label=None, validators=None, coerce=unicode,
+                 choices=None, allow_blank=False, blank_text=None, **kwargs):
+        super(Select2Field, self).__init__(
+            label, validators, coerce, choices, **kwargs
+        )
+        self.allow_blank = allow_blank
+        self.blank_text = blank_text or ' '
+
+    def iter_choices(self):
+        if self.allow_blank:
+            yield (u'__None', self.blank_text, self.data is None)
+
+        for value, label in self.choices:
+            yield (value, label, self.coerce(value) == self.data)
+
+    def process_data(self, value):
+        if value is None:
+            self.data = None
+        else:
+            try:
+                self.data = self.coerce(value)
+            except (ValueError, TypeError):
+                self.data = None
+
+    def process_formdata(self, valuelist):
+        if valuelist:
+            if valuelist[0] == '__None':
+                self.data = None
+            else:
+                try:
+                    self.data = self.coerce(valuelist[0])
+                except ValueError:
+                    raise ValueError(self.gettext(u'Invalid Choice: could not coerce'))
+
+    def pre_validate(self, form):
+        if self.allow_blank and self.data is None:
+            return
+        super(Select2Field, self).pre_validate(form)
+
 
 class DatePickerWidget(widgets.TextInput):
     """
@@ -153,7 +192,9 @@ class RenderTemplateWidget(object):
         kwargs.update({
             'field': field,
             '_gettext': gettext,
-            '_ngettext': ngettext})
+            '_ngettext': ngettext,
+            'h': h,
+        })
 
         template = jinja_env.get_template(self.template)
         return template.render(kwargs)
