@@ -1,3 +1,5 @@
+import warnings
+
 from flask import request, url_for, redirect, flash
 
 from jinja2 import contextfunction
@@ -79,14 +81,15 @@ class BaseModelView(BaseView, ActionsMixin):
         two, you can do something like this::
 
             class MyModelView(BaseModelView):
-                column_formatters = dict(price=lambda c, m, p: m.price*2)
+                column_formatters = dict(price=lambda v, c, m, p: m.price*2)
 
         The Callback function has the prototype::
 
-            def formatter(context, model, name):
-                # context is instance of jinja2.runtime.Context
-                # model is model instance
-                # name is property name
+            def formatter(view, context, model, name):
+                # `view` is current administrative view
+                # `context` is instance of jinja2.runtime.Context
+                # `model` is model instance
+                # `name` is property name
                 pass
     """
 
@@ -117,6 +120,13 @@ class BaseModelView(BaseView, ActionsMixin):
                 column_type_formatters = MY_DEFAULT_FORMATTERS
 
         Type formatters have lower priority than list column formatters.
+
+        The callback function has following prototype::
+
+            def type_formatter(view, value):
+                # `view` is current administrative view
+                # `value` value to format
+                pass
     """
 
     column_labels = ObsoleteAttr('column_labels', 'rename_columns', None)
@@ -167,6 +177,22 @@ class BaseModelView(BaseView, ActionsMixin):
                 column_sortable_list = ('name', ('user', User.username))
     """
 
+    column_default_sort = None
+    """
+        Default sort column if no sorting is applied.
+
+        Example::
+
+            class MyModelView(BaseModelView):
+                column_default_sort = 'user'
+
+        You can use tuple to control ascending descending order. In following example, items
+        will be sorted in descending order::
+
+            class MyModelView(BaseModelView):
+                column_default_sort = ('user', True)
+    """
+
     column_searchable_list = ObsoleteAttr('column_searchable_list',
                                           'searchable_columns',
                                           None)
@@ -180,7 +206,7 @@ class BaseModelView(BaseView, ActionsMixin):
             class MyModelView(BaseModelView):
                 column_searchable_list = ('name', 'email')
     """
-    
+
     column_choices = None
     """
         Map choices to columns in list view
@@ -642,7 +668,7 @@ class BaseModelView(BaseView, ActionsMixin):
         """
         raise NotImplemented('Please implement get_one method')
 
-    # Model handlers
+    # Model event handlers
     def on_model_change(self, form, model):
         """
             Perform some actions after a model is created or updated.
@@ -650,7 +676,30 @@ class BaseModelView(BaseView, ActionsMixin):
             Called from create_model and update_model in the same transaction
             (if it has any meaning for a store backend).
 
-            By default do nothing.
+            By default does nothing.
+
+            :param form:
+                Form used to create/update model
+            :param model:
+                Model that will be created/updated
+        """
+        pass
+
+    def after_model_change(self, form, model, is_created):
+        """
+            Perform some actions after a model was created or updated and
+            committed to the database.
+
+            Called from create_model after successful database commit.
+
+            By default does nothing.
+
+            :param form:
+                Form used to create/update model
+            :param model:
+                Model that was created/updated
+            :param is_created:
+                True if model was created, False if model was updated
         """
         pass
 
@@ -802,6 +851,12 @@ class BaseModelView(BaseView, ActionsMixin):
         """
         return name not in self.action_disallowed_list
 
+    def _get_field_value(self, model, name):
+        """
+            Get unformatted field value from the model
+        """
+        return rec_getattr(model, name)
+
     @contextfunction
     def get_list_value(self, context, model, name):
         """
@@ -816,9 +871,17 @@ class BaseModelView(BaseView, ActionsMixin):
         """
         column_fmt = self.column_formatters.get(name)
         if column_fmt is not None:
-            return column_fmt(context, model, name)
+            try:
+                return column_fmt(self, context, model, name)
+            except TypeError:
+                warnings.warn('Column formatter prototype was changed to accept view as first input parameter.\n' +
+                              'Please update %s %s formatter to accept 4 parameters.' % (self.name, name),
+                              stacklevel=2)
+                self.column_formatters[name] = lambda _, c, m, n: column_fmt(c, m, n)
 
-        value = rec_getattr(model, name)
+                return column_fmt(context, model, name)
+
+        value = self._get_field_value(model, name)
 
         choices_map = self._column_choices_map.get(name, {})
         if choices_map:
@@ -826,7 +889,15 @@ class BaseModelView(BaseView, ActionsMixin):
 
         type_fmt = self.column_type_formatters.get(type(value))
         if type_fmt is not None:
-            value = type_fmt(value)
+            try:
+                value = type_fmt(self, value)
+            except TypeError:
+                warnings.warn('Type formatter prototype was changed to accept view as first input parameter.\n' +
+                              'Please update %s %s formatter to accept 2 parameters.' % (self.name, type(value)),
+                              stacklevel=2)
+                self.column_type_formatters[type(value)] = lambda _, value: type_fmt(value)
+
+                value = type_fmt(value)
 
         return value
 
@@ -980,9 +1051,10 @@ class BaseModelView(BaseView, ActionsMixin):
                 return redirect(return_url)
 
         return self.render(self.edit_template,
-                               form=form,
-                               form_widget_args=self.form_widget_args,
-                               return_url=return_url)
+                           model=model,
+                           form=form,
+                           form_widget_args=self.form_widget_args,
+                           return_url=return_url)
 
     @expose('/delete/', methods=('POST',))
     def delete_view(self):
