@@ -1,7 +1,7 @@
 from jinja2 import Markup
 
 
-def null_formatter(value):
+def null_formatter(view, value):
     """
         Return `NULL` as the string for `None` value
 
@@ -11,7 +11,7 @@ def null_formatter(value):
     return Markup('<i>NULL</i>')
 
 
-def empty_formatter(value):
+def empty_formatter(view, value):
     """
         Return empty string for `None` value
 
@@ -21,7 +21,7 @@ def empty_formatter(value):
     return ''
 
 
-def bool_formatter(value):
+def bool_formatter(view, value):
     """
         Return check icon if value is `True` or empty string otherwise.
 
@@ -31,7 +31,7 @@ def bool_formatter(value):
     return Markup('<i class="icon-ok"></i>' if value else '')
 
 
-def list_formatter(values):
+def list_formatter(view, values):
     """
         Return string with comma separated values
 
